class SwiggyCliQuitError(Exception):
    pass


class SwiggyCliAuthError(Exception):
    pass


class SwiggyCliConfigError(Exception):
    pass


class SwiggyAPIError(Exception):
    pass


class SwiggyDBError(Exception):
    pass
